
#include <stdio.h>

#include <stdlib.h>

#include <unistd.h>

#include <string.h>

#include <sys/types.h>

#include <sys/stat.h>

#include <fcntl.h>

#include <alloca.h>

#include <sys/types.h>

#include "http.h"

#include "server_const.h"

#include "local_time.h"

#include "histo.h"
extern char *chemin_fichiers;
extern struct queue_hist *q_log;

int processHttp (int sockfd, char *ipcli) {
    int rHttpCode, keepAlive;
    struct elem_hist *elemHist = NULL;
    stuHttpData *httpData = (stuHttpData *) calloc (1, sizeof (stuHttpData));
    httpData->socketfd = sockfd;
    httpData->q_ipcli = ipcli;
    readQueryHeader (httpData);
    parseHeader (httpData);
    if ((rHttpCode = fileInfo (httpData)) == 404) {
        httpData->q_filepath = FILE_404;
        if (fileInfo (httpData) != 200) {
            fprintf (stderr, "File 404 not found\n");
            exit (EXIT_FAILURE);
        }
    }
    else if (rHttpCode == 403) {
        httpData->q_filepath = FILE_403;
        if (fileInfo (httpData) != 200) {
            fprintf (stderr, "File 403 not found\n");
            exit (EXIT_FAILURE);
        }
    }
    buildHeader (httpData);
    char *fullUri = (char *) alloca ((1 + sizeof (httpData->q_host) + sizeof (httpData->q_filename)) * sizeof (char));
    sprintf (fullUri, "%s/%s", httpData -> q_host, httpData -> q_filename);
    elemHist = (struct elem_hist *) create_new_elem_hist (fullUri, httpData->q_ipcli, rHttpCode);
    ((struct queue_hist *) q_log)->push (q_log, (void *) elemHist);
    print_queue (q_log);
    sendFile (httpData);
    keepAlive = httpData->q_keep_alive;
    return keepAlive;
}

void *readQueryHeader (stuHttpData *httpData) {
    httpData->q_header = calloc (TAILLE_REQUETE_MAX, sizeof (char));
    char *endHeader = NULL;
    char *btmp;
    int nb;
    int nb_read = 0;
    do {
        btmp = calloc (TAILLE_READ_BUFFER, sizeof (char));
        if ((nb = read (httpData->socketfd, btmp, TAILLE_READ_BUFFER)) < 0) {
            perror ("read");
            exit (EXIT_FAILURE);
        }
        if (nb_read >= TAILLE_REQUETE_MAX)
            break;
        strcat (httpData -> q_header, btmp);
        endHeader = strstr (httpData->q_header, CRLF CRLF);
        free (btmp);
    }
    while (endHeader == NULL);
    return 0;
}

void *parseHeader (stuHttpData *httpData) {
    char *token;
    char *rUri = calloc (TAILLE_READ_BUFFER, sizeof (char));
    httpData->q_keep_alive = 0;
    httpData->q_method = calloc (16, sizeof (char));
    httpData->q_host = calloc (TAILLE_READ_BUFFER, sizeof (char));
    sscanf (httpData -> q_header, "%s %s", httpData -> q_method, rUri);
    token = strstr (httpData->q_header, "Host:");
    sscanf (token, "%*s %s", httpData -> q_host);
    if (strstr (httpData->q_header, "Connection: keep-alive") != NULL) {
        httpData->q_keep_alive = 1;
    }
    if (strcmp (rUri, "/") == 0)
        httpData->q_filename = FILE_INDEX;
    else {
        httpData->q_filename = calloc (strlen (rUri) -1, sizeof (char));
        strcpy (httpData -> q_filename, rUri + 1);
    }
    httpData->q_filepath = calloc (strlen (chemin_fichiers) +strlen (httpData->q_filename), sizeof (char));
    sprintf (httpData -> q_filepath, "%s%s", chemin_fichiers, httpData -> q_filename);
    free (rUri);
    return 0;
}

void *sendFile (stuHttpData *httpData) {
    int fdFile, rd;
    char bufFile [TAILLE_READ_BUFFER];
    if (write (httpData->socketfd, httpData->r_header, httpData->r_header_size) < 0) {
        perror ("Write Header");
        exit (EXIT_FAILURE);
    }
    if ((fdFile = open (httpData->q_filepath, O_RDONLY)) < 0) {
        perror ("open");
        exit (EXIT_FAILURE);
    }
    do {
        if ((rd = read (fdFile, bufFile, TAILLE_READ_BUFFER)) < 0) {
            perror ("read");
            exit (EXIT_FAILURE);
        }
        if ((write (httpData->socketfd, bufFile, rd)) < 0) {
            perror ("Write File");
            exit (EXIT_FAILURE);
        }
    }
    while (rd != 0);
    close (fdFile);
    return 0;
}

void *buildHeader (stuHttpData *httpData) {
    if (httpData->r_code == 200)
        httpData->r_status = "OK";
    else if (httpData->r_code == 403)
        httpData->r_status = "Forbidden";
    else if (httpData->r_code == 404)
        httpData->r_status = "Not found";
    char *tmp_str = (char *) alloca (TAILLE_READ_BUFFER * sizeof (char));
    sprintf (tmp_str, "HTTP/%s %d %s\n" "Server: %s\n" "Content-Length: %d\n\n", HTTP_VERSION, httpData -> r_code, httpData -> r_status, SERVER_INFO, httpData -> r_content_length);
    httpData->r_header_size = strlen (tmp_str);
    httpData->r_header = calloc (httpData->r_header_size + 1, sizeof (char));
    strcpy (httpData -> r_header, tmp_str);
    return 0;
}

int fileInfo (stuHttpData *httpData) {
    struct stat fileStat;
    httpData->r_content_length = 0;
    if (stat (httpData->q_filepath, &fileStat) < 0) {
        fprintf (stderr, "file %s not found\n", httpData -> q_filepath);
        httpData->r_code = 404;
        return 404;
    }
    else {
        httpData->r_content_length = fileStat.st_size;
        httpData->r_code = 200;
        return 200;
    }
}

