
#include <stdio.h>

#include <stdlib.h>

#include <unistd.h>

#include <string.h>

#include <errno.h>

#include <signal.h>

#include <arpa/inet.h>

#include <netdb.h>

#include <netinet/in.h>

#include <sys/types.h>

#include <sys/socket.h>

#include "socket.h"

#include "process_management.h"

#include "server_const.h"

#include "http.h"

#include "histo.h"
extern char *port_srv;
extern char *chemin_fichiers;
extern int taille_log;
extern int max_connexion;
unsigned  int *finalAddress;
struct queue_hist *q_log = NULL;

int *create_socket_stream (const char *host_name, const char *serv_port, const char *proto_name) {
    int *sock;
    int port;
    int status;
    int i;
    int yes = 1;
    int reuse_addr = 1;
    int one_more_time = 4;
    struct hostent *hostent;
    struct servent *servent;
    struct protoent *protoent;
    struct addrinfo hints;
    struct addrinfo *res;
    struct addrinfo *rp;
    memset (& hints, 0, sizeof (struct addrinfo));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;
    hints.ai_protocol = 0;
    sock = calloc (2, sizeof (int));
    if (host_name != NULL)
        if ((hostent = gethostbyname (host_name)) == NULL) {
            perror ("gethostbyname");
            return NULL;
        }
    if ((protoent = getprotobyname (proto_name)) == NULL) {
        perror ("getprotobyname");
        return NULL;
    }
    if (serv_port != NULL) {
        if (sscanf (serv_port, "%d", &port) == 1) {
            if (port < 0) {
                fprintf (stderr, "Le numéro de port doit être un nombre positif.\n");
                return NULL;
            }
            else if (port < NUM_PORT_MIN) {
                if ((servent = getservbyport (htons (port), protoent->p_name)) == NULL) {
                    perror ("getservbyport");
                    return NULL;
                }
            }
            else if (port > NUM_PORT_MAX) {
                fprintf (stderr, "Le numéro de port est trop grand.\n");
                return NULL;
            }
        }
        else {
            perror ("sscanf");
            return NULL;
        }
    }
    if ((status = getaddrinfo (NULL, serv_port, &hints, &res)) != 0) {
        perror ("getaddrinfo");
        fprintf (stderr, "%s\n", gai_strerror (status));
        return NULL;
    }
    for (rp = res, i = 1; rp != NULL; rp = rp->ai_next, i++) {
        fprintf (stdout, "sevice [%d] = %s\n", i, (rp -> ai_family != 2) ? ((rp -> ai_family != 10) ? "UNDEF" : "IPv6") : "IPv4");
    }
    for (rp = res, i = 0; rp != NULL; rp = rp->ai_next, i++) {
        one_more_time = 4;
        if ((sock[i] = socket (rp->ai_family, rp->ai_socktype, rp->ai_protocol)) < 0)
            continue;
        setsockopt (sock [i], SOL_SOCKET, SO_REUSEADDR, & reuse_addr, sizeof (int));
        if (rp->ai_family == AF_INET6)
            setsockopt (sock[i], SOL_IPV6, IPV6_V6ONLY, &yes, sizeof (int));
        do {
            if (bind (sock[i], rp->ai_addr, rp->ai_addrlen) < 0) {
                if (errno == EADDRINUSE) {
                    fprintf (stderr, "L'adresse affecté à la socket est déjà utilisée.\n");
                    fprintf (stderr, "Nouvelle tentative d'Identification de la socket.\n");
                    one_more_time--;
                    usleep (500000);
                }
                else {
                    close (sock [i]);
                    perror ("bind");
                    return NULL;
                }
            }
            else {
                fprintf (stdout, "La famille d'adresse %s a été affecté à la socket sock[%d].\n", (rp -> ai_addr -> sa_family != 2) ? ((rp -> ai_addr -> sa_family != 10) ? "UNDEF" : "IPv6") : "IPv4", i);
                one_more_time = 0;
            }
        }
        while (one_more_time > 0);
    }
    return sock;
}

int tcp_server (void *arg) {
    int *sock_name;
    struct server_process *psp;
    if ((sock_name = create_socket_stream (NULL, ((struct serv_param *) arg)->port_srv, "tcp")) == NULL) {
        fprintf (stderr, "L'appel à create_socket_stream a échoué.\n");
        return -1;
    }
    signal (SIGCHLD, SIG_IGN);
    psp = init_server_process (ipv4_process, ipv6_process, sock_name);
    call_fork (fork (), psp);
    delete_server_process (psp);
    return 0;
}

int close_tcp_server (void) {
    return 0;
}

void process_connection (int sock) {
    char ipcli [TAILLE_READ_BUFFER];
    int keepAlive;
    q_log = new_queue (get_size_queue, get_max_size_queue, push, pop, get_elem, get_nb_elem, get_size_elem, taille_log);
    fprintf (stdout, "Connexion : locale   ");
    print_socket_address (sock, LOCAL, NULL);
    fprintf (stdout, "\t    distante ");
    print_socket_address (sock, DISTANT, ipcli);
    do {
        keepAlive = processHttp (sock, ipcli);
    }
    while (keepAlive);
    close (sock);
}

int print_socket_address (int sock, int where, char *ext_buffer) {
    struct sockaddr_in *addr4 = NULL;
    struct sockaddr_in6 *addr6 = NULL;
    struct sockaddr *addr = NULL;
    socklen_t len;
    char buffer [TAILLE_READ_BUFFER];
    char bufferp [TAILLE_READ_BUFFER];
    int (*getname) (int, struct sockaddr *, socklen_t *);
    addr = calloc (1, sizeof (struct sockaddr));
    len = sizeof (struct sockaddr);
    if (where == 0)
        getname = getsockname;
    else if (where == 1)
        getname = getpeername;
    else {
        fprintf (stderr, "Le paramètre where est erroné.\n");
        return -1;
    }
    if (getname (sock, addr, &len) < 0) {
        perror ("getsockname or getpeername");
        return -1;
    }
    switch (addr->sa_family) {
    case AF_INET :
        len = sizeof (struct sockaddr_in);
        addr4 = calloc (1, sizeof (struct sockaddr_in));
        if (getname (sock, (struct sockaddr *) addr4, &len) < 0) {
            perror ("getsockname");
            return -1;
        }
        inet_ntop (AF_INET, (void *) & (addr4 -> sin_addr), buffer, 256);
        strcpy (bufferp, "IPv4 = ");
        strcat (bufferp, buffer);
        break;
    case AF_INET6 :
        len = sizeof (struct sockaddr_in6);
        addr6 = calloc (1, sizeof (struct sockaddr_in6));
        if (getname (sock, (struct sockaddr *) addr6, &len) < 0) {
            perror ("getsockname");
            return -1;
        }
        inet_ntop (AF_INET6, (void *) & (addr6 -> sin6_addr), buffer, 256);
        strcpy (bufferp, "IPv6 = ");
        strcat (bufferp, buffer);
        break;
    }
    fprintf (stdout, "%s, Port = %u\n", bufferp, (addr4 != NULL) ? ntohs (addr4 -> sin_port) : ntohs (addr6 -> sin6_port));
    if (addr4 != NULL && *finalAddress == -1) {
        *finalAddress = ntohs (addr4->sin_port);
    }
    if (ext_buffer != NULL) {
        strcpy (ext_buffer, buffer);
    }
    if (addr4 != NULL)
        free (addr4);
    if (addr6 != NULL)
        free (addr6);
    free (addr);
    return 0;
}

