struct stuHttpData {
    int socketfd;
    char *q_ipcli;
    char *q_method;
    char *q_host;
    char *q_filename;
    char *q_filepath;
    char *q_header;
    int q_keep_alive;
    char *r_status;
    char *r_date;
    char *r_content_mime;
    char *r_header;
    int r_code;
    int r_content_length;
    int r_header_size;
};
struct elem_hist {
    char *q_url;
    char *q_ipcli;
    char *q_date;
    int q_staterr;
    struct elem_hist *q_next;
};
struct queue_hist {
    int nb_elem;
    long  size_queue;
    long  max_size_queue;
    void *first_elem;
    void *last_elem;
    long  (*get_size_queue) (void *);
    long  (*get_max_size_queue) (void *);
    int (*push) (void *, void *);
    int (*pop) (void *);
    void * (*get_elem) (void *, int);
    int (*get_nb_elem) (void *);
    long  (*get_size_elem) (void *);
};
struct server_process {
    int (*ptr_process [2]) (void *);
    void *data;
};
struct serv_param {
    char *port_srv;
    char *chemin_fichiers;
    int taille_log;
    int max_connexion;
};
struct sigaction *list_action;
unsigned  int *finalAddress;
struct queue_hist *q_log = NULL;
int max_connexion = MAX_CONNEXION_CLIENTS;
int taille_log = TAILLE_FICHIER_LOG;
char *chemin_fichiers = CHEMIN_FICHIERS_HTML;
char *port_srv = PORT_SERVEUR_DEFAUT;

int tcp_server (void *arg) {
    int *sock_name;
    struct server_process *psp;
    if ((sock_name = create_socket_stream (NULL, ((struct serv_param *) arg)->port_srv, "tcp")) == NULL) {
        fprintf (stderr, "L'appel à create_socket_stream a échoué.\n");
        return -1;
    }
    signal (SIGCHLD, SIG_IGN);
    psp = init_server_process (ipv4_process, ipv6_process, sock_name);
    call_fork (fork (), psp);
    delete_server_process (psp);
    return 0;
}

int *create_socket_stream (const char *host_name, const char *serv_port, const char *proto_name) {
    int *sock;
    int port;
    int status;
    int i;
    int yes = 1;
    int reuse_addr = 1;
    int one_more_time = 4;
    struct hostent *hostent;
    struct servent *servent;
    struct protoent *protoent;
    struct addrinfo hints;
    struct addrinfo *res;
    struct addrinfo *rp;
    memset (& hints, 0, sizeof (struct addrinfo));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_PASSIVE;
    hints.ai_protocol = 0;
    sock = calloc (2, sizeof (int));
    if (host_name != NULL)
        if ((hostent = gethostbyname (host_name)) == NULL) {
            perror ("gethostbyname");
            return NULL;
        }
    if ((protoent = getprotobyname (proto_name)) == NULL) {
        perror ("getprotobyname");
        return NULL;
    }
    if (serv_port != NULL) {
        if (sscanf (serv_port, "%d", &port) == 1) {
            if (port < 0) {
                fprintf (stderr, "Le numéro de port doit être un nombre positif.\n");
                return NULL;
            }
            else if (port < NUM_PORT_MIN) {
                if ((servent = getservbyport (htons (port), protoent->p_name)) == NULL) {
                    perror ("getservbyport");
                    return NULL;
                }
            }
            else if (port > NUM_PORT_MAX) {
                fprintf (stderr, "Le numéro de port est trop grand.\n");
                return NULL;
            }
        }
        else {
            perror ("sscanf");
            return NULL;
        }
    }
    if ((status = getaddrinfo (NULL, serv_port, &hints, &res)) != 0) {
        perror ("getaddrinfo");
        fprintf (stderr, "%s\n", gai_strerror (status));
        return NULL;
    }
    for (rp = res, i = 1; rp != NULL; rp = rp->ai_next, i++) {
        fprintf (stdout, "sevice [%d] = %s\n", i, (rp -> ai_family != 2) ? ((rp -> ai_family != 10) ? "UNDEF" : "IPv6") : "IPv4");
    }
    for (rp = res, i = 0; rp != NULL; rp = rp->ai_next, i++) {
        one_more_time = 4;
        if ((sock[i] = socket (rp->ai_family, rp->ai_socktype, rp->ai_protocol)) < 0)
            continue;
        setsockopt (sock [i], SOL_SOCKET, SO_REUSEADDR, & reuse_addr, sizeof (int));
        if (rp->ai_family == AF_INET6)
            setsockopt (sock[i], SOL_IPV6, IPV6_V6ONLY, &yes, sizeof (int));
        do {
            if (bind (sock[i], rp->ai_addr, rp->ai_addrlen) < 0) {
                if (errno == EADDRINUSE) {
                    fprintf (stderr, "L'adresse affecté à la socket est déjà utilisée.\n");
                    fprintf (stderr, "Nouvelle tentative d'Identification de la socket.\n");
                    one_more_time--;
                    usleep (500000);
                }
                else {
                    close (sock [i]);
                    perror ("bind");
                    return NULL;
                }
            }
            else {
                fprintf (stdout, "La famille d'adresse %s a été affecté à la socket sock[%d].\n", (rp -> ai_addr -> sa_family != 2) ? ((rp -> ai_addr -> sa_family != 10) ? "UNDEF" : "IPv6") : "IPv4", i);
                one_more_time = 0;
            }
        }
        while (one_more_time > 0);
    }
    return sock;
}

struct server_process *init_server_process (int (*ptr_child_process) (void *), int (*ptr_father_process) (void *), int *sock) {
    struct server_process *psp;
    if ((psp = calloc (1, sizeof (struct server_process))) == NULL) {
        perror ("calloc");
        return NULL;
    }
    psp->ptr_process[0] = ptr_child_process;
    psp->ptr_process[1] = ptr_father_process;
    psp->data = (void *) sock;
    return psp;
}

int ipv4_process (void *data) {
    struct sockaddr_in addr;
    socklen_t len;
    struct server_process *psp;
    int sock_connected;
    int *sock_name = (int *) data;
    int **sock_process = calloc (2, sizeof (int *));
    sock_process[0] = &sock_name[0];
    sock_process[1] = &sock_connected;
    psp = init_server_process (child_process, father_process, (void *) sock_process);
    listen (sock_name [0], TAILLE_FILE_ECOUTE);
    fprintf (stdout, "Mon adresse >> ");
    if ((print_socket_address (sock_name[0], LOCAL, NULL)) < 0) {
        fprintf (stderr, "L'appel à print_socket_address a échoué.\n");
        return -1;
    }
    while (!close_tcp_server ()) {
        len = sizeof (struct sockaddr_in);
        if ((sock_connected = accept (sock_name[0], (struct sockaddr *) &addr, &len)) < 0) {
            perror ("accept");
            return -1;
        }
        call_fork (fork (), psp);
    }
    delete_server_process (psp);
    close (sock_name [0]);
    return EXIT_SUCCESS;
}

int child_process (void *data) {
    int **sock = (int **) data;
    close (* (sock [0]));
    process_connection (* (sock [1]));
    exit (EXIT_SUCCESS);
}

void process_connection (int sock) {
    char ipcli [TAILLE_READ_BUFFER];
    int keepAlive;
    q_log = new_queue (get_size_queue, get_max_size_queue, push, pop, get_elem, get_nb_elem, get_size_elem, taille_log);
    fprintf (stdout, "Connexion : locale   ");
    print_socket_address (sock, LOCAL, NULL);
    fprintf (stdout, "\t    distante ");
    print_socket_address (sock, DISTANT, ipcli);
    do {
        keepAlive = processHttp (sock, ipcli);
    }
    while (keepAlive);
    close (sock);
}

void *new_queue (long  (*get_size_queue) (void *), long  (*get_max_size_queue) (void *), int (*push) (void *, void *), int (*pop) (void *), void * (*get_elem) (void *, int), int (*get_nb_elem) (void *), long  (*get_size_elem) (void *), long  max_size_queue) {
    struct queue_hist *queue = NULL;
    if ((queue = calloc (1, sizeof (struct queue_hist))) == NULL) {
        perror ("calloc");
        return NULL;
    }
    queue->nb_elem = 0;
    queue->size_queue = 0;
    queue->max_size_queue = max_size_queue;
    queue->first_elem = NULL;
    queue->last_elem = NULL;
    queue->get_size_queue = get_size_queue;
    queue->get_max_size_queue = get_max_size_queue;
    queue->push = push;
    queue->pop = pop;
    queue->get_elem = get_elem;
    queue->get_nb_elem = get_nb_elem;
    queue->get_size_elem = get_size_elem;
    return (void *) queue;
}

long  get_size_queue (void *q_this) {
    if (q_this != NULL)
        return ((struct queue_hist *) q_this)->size_queue;
    return -1;
}

long  get_max_size_queue (void *q_this) {
    if (q_this != NULL)
        return ((struct queue_hist *) q_this)->max_size_queue;
    return -1;
}

int push (void *q_this, void *q_elem) {
    int size_elem = 0;
    void *q_elem_tmp = NULL;
    if ((q_this != NULL) && (q_elem != NULL)) {
        size_elem = get_size_elem (q_elem);
        while ((((struct queue_hist *) q_this)->get_size_queue (q_this) + size_elem) > ((struct queue_hist *) q_this)->get_max_size_queue (q_this)) {
            fprintf (stdout, "\n\nTaille trop petite\n\n");
            ((struct queue_hist *) q_this)->pop (q_this);
        }
        ((struct queue_hist *) q_this)->nb_elem++;
        ((struct queue_hist *) q_this)->size_queue += size_elem;
        if ((((struct queue_hist *) q_this)->first_elem == NULL) && (((struct queue_hist *) q_this)->last_elem == NULL)) {
            ((struct queue_hist *) q_this)->first_elem = q_elem;
            ((struct queue_hist *) q_this)->last_elem = q_elem;
        }
        else {
            q_elem_tmp = ((struct queue_hist *) q_this)->last_elem;
            ((struct queue_hist *) q_this)->last_elem = q_elem;
            ((struct elem_hist *) q_elem)->q_next = q_elem_tmp;
        }
        return EXIT_SUCCESS;
    }
    return -1;
}

long  get_size_elem (void *q_elem) {
    long  size_elem = 0;
    if (q_elem != NULL) {
        if (((struct elem_hist *) q_elem)->q_url != NULL) {
            for (int i = 0; ((struct elem_hist *) q_elem)->q_url[i] != 0; i++)
                size_elem += sizeof (char);
            size_elem += sizeof (char);
        }
        if (((struct elem_hist *) q_elem)->q_ipcli != NULL) {
            for (int i = 0; ((struct elem_hist *) q_elem)->q_ipcli[i] != 0; i++)
                size_elem += sizeof (char);
            size_elem += sizeof (char);
        }
        if (((struct elem_hist *) q_elem)->q_date != NULL) {
            for (int i = 0; ((struct elem_hist *) q_elem)->q_date[i] != 0; i++)
                size_elem += sizeof (char);
            size_elem += sizeof (char);
        }
        size_elem += sizeof (int);
        size_elem += sizeof (void *);
        return size_elem;
    }
    return -1;
}

int pop (void *q_this) {
    int size_first_elem = 0;
    void *q_elem_tmp = NULL;
    void *q_elem_last_tmp = NULL;
    if (q_this != NULL) {
        size_first_elem = ((struct queue_hist *) q_this)->get_size_elem (((struct queue_hist *) q_this)->first_elem);
        if (((struct queue_hist *) q_this)->nb_elem < 1) {
            return -1;
        }
        else if (((struct queue_hist *) q_this)->nb_elem == 1) {
            q_elem_tmp = ((struct queue_hist *) q_this)->first_elem;
            ((struct queue_hist *) q_this)->first_elem = NULL;
            ((struct queue_hist *) q_this)->last_elem = NULL;
        }
        else if (((struct queue_hist *) q_this)->nb_elem > 1) {
            q_elem_last_tmp = ((struct queue_hist *) q_this)->last_elem;
            q_elem_tmp = ((struct queue_hist *) q_this)->first_elem;
            for (int i = 0; i < (((struct queue_hist *) q_this)->nb_elem - 2); i++) {
                q_elem_last_tmp = ((struct elem_hist *) q_elem_last_tmp)->q_next;
            }
            ((struct elem_hist *) q_elem_last_tmp)->q_next = NULL;
            ((struct queue_hist *) q_this)->first_elem = q_elem_last_tmp;
        }
        ((struct queue_hist *) q_this)->nb_elem--;
        ((struct queue_hist *) q_this)->size_queue -= size_first_elem;
        delete_elem_hist (q_elem_tmp);
        return EXIT_SUCCESS;
    }
    return -1;
}

void delete_elem_hist (void *q_elem) {
    struct elem_hist *tmp_elem = (struct elem_hist *) q_elem;
    free (tmp_elem -> q_url);
    free (tmp_elem -> q_ipcli);
    free (tmp_elem -> q_date);
    free (q_elem);
    tmp_elem->q_next = NULL;
    tmp_elem = NULL;
}

void *get_elem (void *q_this, int index) {
    struct queue_hist *q = (struct queue_hist *) q_this;
    struct elem_hist *e = (struct elem_hist *) (q->last_elem);
    for (int i = 0; i < index; i++) {
        e = (struct elem_hist *) (((struct elem_hist *) e)->q_next);
    }
    return (void *) e;
}

int get_nb_elem (void *q_this) {
    if (q_this != NULL)
        return ((struct queue_hist *) q_this)->nb_elem;
    return -1;
}

int print_socket_address (int sock, int where, char *ext_buffer) {
    struct sockaddr_in *addr4 = NULL;
    struct sockaddr_in6 *addr6 = NULL;
    struct sockaddr *addr = NULL;
    socklen_t len;
    char buffer [TAILLE_READ_BUFFER];
    char bufferp [TAILLE_READ_BUFFER];
    int (*getname) (int, struct sockaddr *, socklen_t *);
    addr = calloc (1, sizeof (struct sockaddr));
    len = sizeof (struct sockaddr);
    if (where == 0)
        getname = getsockname;
    else if (where == 1)
        getname = getpeername;
    else {
        fprintf (stderr, "Le paramètre where est erroné.\n");
        return -1;
    }
    if (getname (sock, addr, &len) < 0) {
        perror ("getsockname or getpeername");
        return -1;
    }
    switch (addr->sa_family) {
    case AF_INET :
        len = sizeof (struct sockaddr_in);
        addr4 = calloc (1, sizeof (struct sockaddr_in));
        if (getname (sock, (struct sockaddr *) addr4, &len) < 0) {
            perror ("getsockname");
            return -1;
        }
        inet_ntop (AF_INET, (void *) & (addr4 -> sin_addr), buffer, 256);
        strcpy (bufferp, "IPv4 = ");
        strcat (bufferp, buffer);
        break;
    case AF_INET6 :
        len = sizeof (struct sockaddr_in6);
        addr6 = calloc (1, sizeof (struct sockaddr_in6));
        if (getname (sock, (struct sockaddr *) addr6, &len) < 0) {
            perror ("getsockname");
            return -1;
        }
        inet_ntop (AF_INET6, (void *) & (addr6 -> sin6_addr), buffer, 256);
        strcpy (bufferp, "IPv6 = ");
        strcat (bufferp, buffer);
        break;
    }
    fprintf (stdout, "%s, Port = %u\n", bufferp, (addr4 != NULL) ? ntohs (addr4 -> sin_port) : ntohs (addr6 -> sin6_port));
    if (addr4 != NULL && *finalAddress == -1) {
        *finalAddress = ntohs (addr4->sin_port);
    }
    if (ext_buffer != NULL) {
        strcpy (ext_buffer, buffer);
    }
    if (addr4 != NULL)
        free (addr4);
    if (addr6 != NULL)
        free (addr6);
    free (addr);
    return 0;
}

int processHttp (int sockfd, char *ipcli) {
    int rHttpCode, keepAlive;
    struct elem_hist *elemHist = NULL;
    stuHttpData *httpData = (stuHttpData *) calloc (1, sizeof (stuHttpData));
    httpData->socketfd = sockfd;
    httpData->q_ipcli = ipcli;
    readQueryHeader (httpData);
    parseHeader (httpData);
    if ((rHttpCode = fileInfo (httpData)) == 404) {
        httpData->q_filepath = FILE_404;
        if (fileInfo (httpData) != 200) {
            fprintf (stderr, "File 404 not found\n");
            exit (EXIT_FAILURE);
        }
    }
    else if (rHttpCode == 403) {
        httpData->q_filepath = FILE_403;
        if (fileInfo (httpData) != 200) {
            fprintf (stderr, "File 403 not found\n");
            exit (EXIT_FAILURE);
        }
    }
    buildHeader (httpData);
    char *fullUri = (char *) alloca ((1 + sizeof (httpData->q_host) + sizeof (httpData->q_filename)) * sizeof (char));
    sprintf (fullUri, "%s/%s", httpData -> q_host, httpData -> q_filename);
    elemHist = (struct elem_hist *) create_new_elem_hist (fullUri, httpData->q_ipcli, rHttpCode);
    ((struct queue_hist *) q_log)->push (q_log, (void *) elemHist);
    print_queue (q_log);
    sendFile (httpData);
    keepAlive = httpData->q_keep_alive;
    return keepAlive;
}

void *readQueryHeader (stuHttpData *httpData) {
    httpData->q_header = calloc (TAILLE_REQUETE_MAX, sizeof (char));
    char *endHeader = NULL;
    char *btmp;
    int nb;
    int nb_read = 0;
    do {
        btmp = calloc (TAILLE_READ_BUFFER, sizeof (char));
        if ((nb = read (httpData->socketfd, btmp, TAILLE_READ_BUFFER)) < 0) {
            perror ("read");
            exit (EXIT_FAILURE);
        }
        if (nb_read >= TAILLE_REQUETE_MAX)
            break;
        strcat (httpData -> q_header, btmp);
        endHeader = strstr (httpData->q_header, CRLF CRLF);
        free (btmp);
    }
    while (endHeader == NULL);
    return 0;
}

void *parseHeader (stuHttpData *httpData) {
    char *token;
    char *rUri = calloc (TAILLE_READ_BUFFER, sizeof (char));
    httpData->q_keep_alive = 0;
    httpData->q_method = calloc (16, sizeof (char));
    httpData->q_host = calloc (TAILLE_READ_BUFFER, sizeof (char));
    sscanf (httpData -> q_header, "%s %s", httpData -> q_method, rUri);
    token = strstr (httpData->q_header, "Host:");
    sscanf (token, "%*s %s", httpData -> q_host);
    if (strstr (httpData->q_header, "Connection: keep-alive") != NULL) {
        httpData->q_keep_alive = 1;
    }
    if (strcmp (rUri, "/") == 0)
        httpData->q_filename = FILE_INDEX;
    else {
        httpData->q_filename = calloc (strlen (rUri) -1, sizeof (char));
        strcpy (httpData -> q_filename, rUri + 1);
    }
    httpData->q_filepath = calloc (strlen (chemin_fichiers) +strlen (httpData->q_filename), sizeof (char));
    sprintf (httpData -> q_filepath, "%s%s", chemin_fichiers, httpData -> q_filename);
    free (rUri);
    return 0;
}

int fileInfo (stuHttpData *httpData) {
    struct stat fileStat;
    httpData->r_content_length = 0;
    if (stat (httpData->q_filepath, &fileStat) < 0) {
        fprintf (stderr, "file %s not found\n", httpData -> q_filepath);
        httpData->r_code = 404;
        return 404;
    }
    else {
        httpData->r_content_length = fileStat.st_size;
        httpData->r_code = 200;
        return 200;
    }
}

void *buildHeader (stuHttpData *httpData) {
    if (httpData->r_code == 200)
        httpData->r_status = "OK";
    else if (httpData->r_code == 403)
        httpData->r_status = "Forbidden";
    else if (httpData->r_code == 404)
        httpData->r_status = "Not found";
    char *tmp_str = (char *) alloca (TAILLE_READ_BUFFER * sizeof (char));
    sprintf (tmp_str, "HTTP/%s %d %s\n" "Server: %s\n" "Content-Length: %d\n\n", HTTP_VERSION, httpData -> r_code, httpData -> r_status, SERVER_INFO, httpData -> r_content_length);
    httpData->r_header_size = strlen (tmp_str);
    httpData->r_header = calloc (httpData->r_header_size + 1, sizeof (char));
    strcpy (httpData -> r_header, tmp_str);
    return 0;
}

void *create_new_elem_hist (char *url, char *ipcli, int staterr) {
    struct elem_hist *elem = NULL;
    if ((elem = calloc (1, sizeof (struct elem_hist))) == NULL) {
        perror ("calloc");
        return NULL;
    }
    if ((elem->q_url = calloc (strlen (url) +1, sizeof (char))) == NULL) {
        perror ("calloc");
        return NULL;
    }
    strcpy (elem -> q_url, url);
    if ((elem->q_ipcli = calloc (strlen (ipcli) +1, sizeof (char))) == NULL) {
        perror ("calloc");
        return NULL;
    }
    strcpy (elem -> q_ipcli, ipcli);
    elem->q_staterr = staterr;
    if (get_local_time (&(elem->q_date), 0) == -1) {
        fprintf (stderr, "L'appel à get_local_time a échoué.\n");
        return NULL;
    }
    elem->q_next = NULL;
    return (void *) elem;
}

int get_local_time (char **buffer, int mode) {
    char buf_tmp [50];
    int size;
    time_t temps;
    struct tm *tm;
    if ((time (&temps)) == (time_t) -1) {
        perror ("time");
        return -1;
    }
    if ((tm = localtime (&temps)) == NULL) {
        perror ("localtime");
        return -1;
    }
    if (mode == 0)
        sprintf (buf_tmp, "[localtime = %02d/%02d/%02d - %02d:%02d:%02d - (%s)]", tm->tm_mday, tm->tm_mon + 1, tm->tm_year % 100, tm->tm_hour, tm->tm_min, tm->tm_sec, (tm->tm_isdst > 0) ? "Eté" : (tm->tm_isdst == 0) ? "Hiver" : "Undef");
    else if (mode == 1)
        sprintf (buf_tmp, "%02d%02d%02d_%02d%02d%02d", tm->tm_year % 100, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
    assert (* buffer == NULL);
    size = strlen (buf_tmp);
    if ((*buffer = calloc (size +1, sizeof (char))) == NULL) {
        perror ("calloc");
        return -1;
    }
    strcpy (* buffer, buf_tmp);
    return size;
}

void print_queue (void *q_this) {
    struct queue_hist *q = (struct queue_hist *) q_this;
    struct elem_hist *e = (struct elem_hist *) (q->last_elem);
    fprintf (stdout, "\nprint queue @ = %p\n-----------\n", q_this);
    for (int i = 0; i < q->get_nb_elem (q_this); i++) {
        print_elem ((void *) e);
        e = (struct elem_hist *) (((struct elem_hist *) e)->q_next);
    }
}

void print_elem (void *q_elem) {
    struct elem_hist *e = (struct elem_hist *) q_elem;
    fprintf (stdout, "@@@@@\nsizeof() = %ld [o]\n", get_size_elem (q_elem));
    fprintf (stdout, "+++++\nadr : %p\n+++++\nurl : %s\nipcli : %s\ndate : %s\nstaterr : %d\nq_next : %p\n@@@@@\n", (void *) e, e -> q_url, e -> q_ipcli, e -> q_date, e -> q_staterr, (void *) (e -> q_next));
}

void *sendFile (stuHttpData *httpData) {
    int fdFile, rd;
    char bufFile [TAILLE_READ_BUFFER];
    if (write (httpData->socketfd, httpData->r_header, httpData->r_header_size) < 0) {
        perror ("Write Header");
        exit (EXIT_FAILURE);
    }
    if ((fdFile = open (httpData->q_filepath, O_RDONLY)) < 0) {
        perror ("open");
        exit (EXIT_FAILURE);
    }
    do {
        if ((rd = read (fdFile, bufFile, TAILLE_READ_BUFFER)) < 0) {
            perror ("read");
            exit (EXIT_FAILURE);
        }
        if ((write (httpData->socketfd, bufFile, rd)) < 0) {
            perror ("Write File");
            exit (EXIT_FAILURE);
        }
    }
    while (rd != 0);
    close (fdFile);
    return 0;
}

int father_process (void *data) {
    int **sock = (int **) data;
    close (* (sock [1]));
    return EXIT_SUCCESS;
}

int close_tcp_server (void) {
    return 0;
}

int call_fork (int val, struct server_process *ptr_sp) {
    if (val < 0)
        return -1;
    (*ptr_sp->ptr_process[((!val) ? 0 : 1)]) ((void *) ptr_sp->data);
    return EXIT_SUCCESS;
}

void delete_server_process (struct server_process *ptr_sp) {
    free (ptr_sp);
}

int ipv6_process (void *data) {
    struct sockaddr_in addr;
    socklen_t len;
    struct server_process *psp;
    int sock_connected;
    int *sock_name = (int *) data;
    int **sock_process = calloc (2, sizeof (int *));
    sock_process[0] = &sock_name[1];
    sock_process[1] = &sock_connected;
    psp = init_server_process (child_process, father_process, (void *) sock_process);
    listen (sock_name [1], TAILLE_FILE_ECOUTE);
    fprintf (stdout, "Mon adresse >> ");
    if ((print_socket_address (sock_name[1], LOCAL, NULL)) < 0) {
        fprintf (stderr, "L'appel à print_socket_address a échoué.\n");
        return -1;
    }
    while (!close_tcp_server ()) {
        len = sizeof (struct sockaddr_in);
        if ((sock_connected = accept (sock_name[1], (struct sockaddr *) &addr, &len)) < 0) {
            perror ("accept");
            return -1;
        }
        call_fork (fork (), psp);
    }
    delete_server_process (psp);
    close (sock_name [1]);
    return EXIT_SUCCESS;
}

void handler (int num) {
    switch (num) {
    case SIGINT :
        fprintf (stdout, "Processus %d : signal SIGINT\nInterruption du processus\n", getpid ());
        if (q_log != NULL)
            delete_queue (q_log);
        delete_handler (list_action);
        exit (EXIT_SUCCESS);
    default :
        break;
    }
}

void delete_queue (void *q_this) {
    int nb_elem = ((struct queue_hist *) q_this)->nb_elem;
    for (int i = 0; i < nb_elem; i++) {
        ((struct queue_hist *) q_this)->pop (q_this);
    }
    ((struct queue_hist *) q_this)->first_elem = NULL;
    ((struct queue_hist *) q_this)->last_elem = NULL;
    free (q_this);
}

void delete_handler (struct sigaction *list_action) {
    free (list_action);
}

